/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.icp.telecom.reference.dynamicorderfulfillment.mock;

import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBFactoryConstants;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.icp.industry.db.manager.WicpDBFactory;
import com.ibm.websphere.icp.telecom.reference.dynamicorderfulfillment.Customermaster;
import com.ibm.websphere.icp.telecom.reference.dynamicorderfulfillment.Serviceorder;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

/**
 * @author nikunj.panchal
 * 
 */

public class SubmitServiceOrderImpl {
	/**
	 * Default constructor.
	 */
	public SubmitServiceOrderImpl() {
		super();
	}

	/**
	 * Return a reference to the component service instance for this
	 * implementation class. This method should be used when passing this
	 * service to a partner reference or if you want to invoke this component
	 * service asynchronously.
	 * 
	 * @generated (com.ibm.wbit.java)
	 */
	@SuppressWarnings("unused")
	private Object getMyService() {
		return (Object) ServiceManager.INSTANCE.locateService("self");
	}

	/**
	 * Method generated to support implementation of operation
	 * "submitServiceOrder" defined for WSDL port type named
	 * "SubmitServiceOrderPT".
	 * 
	 * The presence of commonj.sdo.DataObject as the return type and/or as a
	 * parameter type conveys that it is a complex type. Please refer to the
	 * WSDL Definition for more information on the type of input, output and
	 * fault(s).
	 */
	public DataObject submitServiceOrder(DataObject aServiceOrderMessage) {
		DataObject resServiceOrderMessage = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/fabric/2009/12/telecom/operations/fulfillment/schema/ServiceOrderMessage",
						"ServiceOrderMessage");
		DataObject resServiceOrder = resServiceOrderMessage
				.createDataObject("serviceOrder");

		final Serviceorder serviceOrder = new Serviceorder();
		/**
		 * Generate the ServiceOrderId
		 */
		int randomIntegerForServiceOrderId = (int) (100000000 * Math.random());
		String serviceOrderId = String.valueOf(randomIntegerForServiceOrderId);
		DataObject serviceOrderDO = aServiceOrderMessage
				.getDataObject("serviceOrder");

		/**
		 * Get the ServiceId
		 */
		String serviceId = serviceOrderDO.getString("Id");
		/**
		 * Get The Service ServiceBeginDate and ServiceEndDate
		 */

		Date serviceBeginDate = serviceOrderDO.getDate("RequestedDate");
		Date serviceEndDate = serviceOrderDO.getDate("ExpectedDeliveryDate");

		/**
		 * Get the Status of the service order
		 */
		String serviceStatus = serviceOrderDO.getString("InteractionStatus");

		/**
		 * Get The CustomerId
		 */
		DataObject specification = serviceOrderDO
				.getDataObject("Specification");
		String customerId = specification.getString("Id");

		/**
		 * Create the Map to fetch Customer from Db
		 */
		Map<String, Object> mapParams = new HashMap<String, Object>();
		mapParams.put("id", Integer.valueOf(customerId));

		/**
		 * Set the Service Order Object
		 */

		serviceOrder.setServiceId(Integer.valueOf(serviceId));
		serviceOrder.setBeginDate(new Timestamp(serviceBeginDate.getTime()));
		serviceOrder.setEndDate(new Timestamp(serviceEndDate.getTime()));
		// serviceOrder.setId(serviceOrderId);
		serviceOrder.setStatus(serviceStatus);

		/**
		 * get the DbFactory and persist data
		 */
		Properties dbProperties = new Properties();
		dbProperties.setProperty(IWicpDBFactoryConstants.TYPE, "JPA");
		dbProperties.setProperty(IWicpDBFactoryConstants.PERSISTENCE_UNIT_NAME,
				"DynamicOrderFulfillmentDBAccess");
		IWicpDBManager wicpDbManager = WicpDBFactory.getDBManager(dbProperties);

		/**
		 * Fetch The Customer record from DB
		 */
		Customermaster customermaster = new Customermaster();
		customermaster = (Customermaster) wicpDbManager.executeNamedQuery(
				"findCustomerById", mapParams).get(0);
		serviceOrder.setCustomerMaster(customermaster);

		try {
			wicpDbManager.save(serviceOrder);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		resServiceOrder.setString("Id", String.valueOf(serviceOrder.getId()));
		resServiceOrder.setDate("RequestedDate", serviceOrderDO
				.getDate("RequestedDate"));
		resServiceOrder.setDate("ExpectedDeliveryDate", serviceOrderDO
				.getDate("ExpectedDeliveryDate"));
		resServiceOrder.setString("InteractionStatus", serviceOrderDO
				.getString("InteractionStatus"));
		resServiceOrder.setDataObject("Specification", serviceOrderDO
				.getDataObject("Specification"));

		return resServiceOrderMessage;
	}
}
