/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.icp.telecom.reference.dynamicorderfulfillment.mock;

import java.util.List;
import java.util.Properties;

import com.ibm.websphere.icp.industry.db.manager.IWicpDBFactoryConstants;
import com.ibm.websphere.icp.industry.db.manager.IWicpDBManager;
import com.ibm.websphere.icp.industry.db.manager.WicpDBFactory;
import com.ibm.websphere.icp.telecom.reference.dynamicorderfulfillment.Customeraccount;
import com.ibm.websphere.icp.telecom.reference.dynamicorderfulfillment.Customermaster;
import com.ibm.websphere.icp.telecom.reference.util.ValuesFromCharacteristics;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class UpdateCustomerImpl {
	/**
	 * Default constructor.
	 */
	public UpdateCustomerImpl() {
		super();
	}

	/**
	 * Return a reference to the component service instance for this
	 * implementation class. This method should be used when passing this
	 * service to a partner reference or if you want to invoke this component
	 * service asynchronously.
	 * 
	 * @generated (com.ibm.wbit.java)
	 */
	@SuppressWarnings("unused")
	private Object getMyService() {
		return (Object) ServiceManager.INSTANCE.locateService("self");
	}

	/**
	 * Method generated to support implementation of operation "updateCustomer"
	 * defined for WSDL port type named "UpdateCustomerPT".
	 * 
	 * The presence of commonj.sdo.DataObject as the return type and/or as a
	 * parameter type conveys that it is a complex type. Please refer to the
	 * WSDL Definition for more information on the type of input, output and
	 * fault(s).
	 */
	public DataObject updateCustomer(DataObject aCustomerMessage) {
		final Customeraccount customerAccount = new Customeraccount();
		int accountId;

		final Customermaster customerMaster = new Customermaster();

		DataObject customer = aCustomerMessage.getDataObject("customer");

		/**
		 * Update Customer Record if the User Selected To Update Personal Record
		 */
		if (customer.getString("Status").equalsIgnoreCase("true")) {
			/**
			 * Set The Customer Address Information into the Characteristic
			 * Value
			 */
			DataObject party = customer.getDataObject("Party");

			/***
			 * Set AccountId Into SID.Customer.Specification.Id
			 */
			DataObject specification = party.getDataObject("Specification");
			accountId = Integer.valueOf(specification.getString("Id"));
			customerAccount.setId(accountId);
			customerAccount.setCreditScore(String.valueOf(5000L));

			customerMaster.setAccount(customerAccount);
			customerMaster.setId(Integer.valueOf(party.getString("Id")));

			List characteristicValue = (List) party
					.getList("CharacteristicValue");

			customerMaster.setTitle(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "Title").get(0));
			customerMaster.setFirstName(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "FirstName").get(0));
			customerMaster.setMiddleName(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "MiddleName").get(0));
			customerMaster.setLastName(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "LastName").get(0));
			customerMaster.setStreetNr(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "StreetNumber").get(0));
			customerMaster.setStreetName(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "StreetName").get(0));
			customerMaster.setStreetType(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "Locality").get(0));
			customerMaster.setUnitType(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "AppartmentName").get(0));
			customerMaster.setUnitNr(Integer.valueOf(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "AppartmentUnit").get(0)));
			customerMaster.setCity(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "City").get(0));
			customerMaster.setState(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "State").get(0));
			customerMaster.setCountry(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "Country").get(0));
			customerMaster.setPostCode(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValue, "Zipcode").get(0));

			/**
			 * Set the Alternate Information Into the
			 * SID.CustomerMessage.Customer.ContactMedium.CharacteristicValue
			 */

			List contactMediumList = (List) party.getList("ContactMedium");
			DataObject contactMedium = (DataObject) contactMediumList.get(0);
			List characteristicValueAlternateInfo = (List) contactMedium
					.getList("CharacteristicValue");
			customerMaster.setMobilePhoneNumber(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueAlternateInfo, "MobileNumber")
					.get(0));
			customerMaster.setFixedLinePhoneNumber(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueAlternateInfo, "PhoneNumber")
					.get(0));

			customerMaster.setEmail(ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueAlternateInfo, "Email").get(0));

			/**
			 * get the DbFactory and persist data
			 */
			Properties dbProperties = new Properties();
			dbProperties.setProperty(IWicpDBFactoryConstants.TYPE, "JPA");
			dbProperties.setProperty(
					IWicpDBFactoryConstants.PERSISTENCE_UNIT_NAME,
					"DynamicOrderFulfillmentDBAccess");

			IWicpDBManager wicpDbManager = WicpDBFactory
					.getDBManager(dbProperties);

			try {
				wicpDbManager.update(customerMaster);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		return aCustomerMessage;
	}
}
