/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.icp.telecom.reference.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.websphere.bo.BOFactory;
import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class CharacteristicUtils {

	public static List characteristicsToSIDCharacteristics(
			List<DataObject> characteristics) {

		try {
			BOFactory boFactory = (BOFactory) ServiceManager.INSTANCE
					.locateService("com/ibm/websphere/bo/BOFactory");
			List SIDCharacteristicValues = new ArrayList();

			Iterator iteratorChars = characteristics.iterator();

			DataObject SIDCharacteristicValue;
			DataObject SIDCharacteristic;
			String charName;
			List charValues;

			while (iteratorChars.hasNext()) {
				DataObject characteristic = (DataObject) iteratorChars.next();
				charName = characteristic.getString("name");
				charValues = characteristic.getList("value");

				SIDCharacteristic = IndustrySupportUtil.createDO(
						"http://www.ibm.com/telecom/common/schema/base/v2_0",
						"Characteristic");
				SIDCharacteristic.setString("Name", charName);

				SIDCharacteristicValue = IndustrySupportUtil.createDO(
						"http://www.ibm.com/telecom/common/schema/base/v2_0",
						"CharacteristicValue");
				SIDCharacteristicValue.set("Characteristic", SIDCharacteristic);
				SIDCharacteristicValue.setList("Value", charValues);

				SIDCharacteristicValues.add(SIDCharacteristicValue);
			}

			return SIDCharacteristicValues;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static DataObject SIDcharacteristicsToCharacteristics(
			List<DataObject> SIDCharacteristicValues) {

		try {
			BOFactory boFactory = (BOFactory) ServiceManager.INSTANCE
					.locateService("com/ibm/websphere/bo/BOFactory");

			DataObject characteristics = IndustrySupportUtil.createDO(
					"http://LibraryForOrderFulfillmentMediations",
					"ArrayOfCharacteristic");
			List listOfCharacteristics = new ArrayList();
			Iterator iterator = SIDCharacteristicValues.iterator();
			String name = null;
			List values;
			DataObject characteristic;
			while (iterator.hasNext()) {
				DataObject characteristicValue = (DataObject) iterator.next();
				name = characteristicValue.getString("Characteristic/Name");
				values = characteristicValue.getList("Value");

				characteristic = IndustrySupportUtil.createDO(
						"http://LibraryForOrderFulfillmentMediations",
						"Characteristic");
				characteristic.setString("name", name);
				characteristic.setList("value", values);
				listOfCharacteristics.add(characteristic);
			}
			characteristics.setList("characteristic", listOfCharacteristics);

			return characteristics;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static void appendSIDCharacteristicsToDataObject(DataObject target,
			List<DataObject> characteristicsToAppend1,
			List<DataObject> characteristicsToAppend2) {
		if (characteristicsToAppend1 != null)
			target.getList("CharacteristicValue").addAll(
					characteristicsToAppend1);
		if (characteristicsToAppend2 != null)
			target.getList("CharacteristicValue").addAll(
					characteristicsToAppend2);
	}

	public static List<DataObject> filterCharacteristics(String[] prefixes,
			List<DataObject> characteristicValues) {
		try {
			List<DataObject> filteredCharacteristics = new ArrayList<DataObject>();
			Iterator<DataObject> iterator = characteristicValues.iterator();
			while (iterator.hasNext()) {
				DataObject characteristicValue = (DataObject) iterator.next();
				DataObject characteristic = (DataObject) characteristicValue
						.get("Characteristic");
				String name = characteristic.getString("Name");
				// if(matchingName.equals(name)) {
				for (int i = 0; i < prefixes.length; i++) {
					if (name.startsWith(prefixes[i] + "_")) {
						// values = (List<String>)
						// characteristicValue.getList("Value");
						filteredCharacteristics.add(characteristicValue);
					}
				}
			}
			return filteredCharacteristics;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static DataObject createSIDCharacteristic(String name, String value) {
		try {
			BOFactory boFactory = (BOFactory) ServiceManager.INSTANCE
					.locateService("com/ibm/websphere/bo/BOFactory");
			DataObject SIDCharacteristicValue = IndustrySupportUtil.createDO(
					"http://www.ibm.com/telecom/common/schema/base/v2_0",
					"CharacteristicValue");
			DataObject SIDCharacteristic = IndustrySupportUtil.createDO(
					"http://www.ibm.com/telecom/common/schema/base/v2_0",
					"Characteristic");
			List<String> values = new ArrayList<String>();

			SIDCharacteristic.setString("Name", name);
			SIDCharacteristicValue.set("Characteristic", SIDCharacteristic);
			values.add(value);
			SIDCharacteristicValue.setList("Value", values);

			return SIDCharacteristicValue;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
}
