/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.icp.telecom.reference.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import commonj.sdo.DataObject;

public class TelecomCommonUtil {

	public static DataObject mapSearchParameters(DataObject searchCustomer) {

		DataObject custInfo = searchCustomer.getDataObject("customerInfo");
		DataObject productOrderMessage = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/fabric/2009/12/telecom/operations/fulfillment/schema/ProductOrderMessage",
						"ProductOrderMessage");
		DataObject productOrder = productOrderMessage
				.createDataObject("productOrder");
		DataObject partyRole = IndustrySupportUtil.createDO(
				"http://www.ibm.com/telecom/common/schema/party/v3_0",
				"PartyRole");
		DataObject party = partyRole.createDataObject("Party");

		List characteristicValueList = new ArrayList();

		DataObject characteristicValue;
		if (custInfo.getString("firstName") != null) {
			characteristicValue = putValueInCharacterList("FirstName", custInfo
					.getString("firstName"));
			characteristicValueList.add(characteristicValue);
		}
		party.setList("CharacteristicValue", characteristicValueList);
		if (custInfo.getString("lastName") != null) {
			characteristicValue = putValueInCharacterList("LastName", custInfo
					.getString("lastName"));
			characteristicValueList.add(characteristicValue);
		}
		if (custInfo.getString("middleName") != null) {
			characteristicValue = putValueInCharacterList("MiddleName",
					custInfo.getString("middleName"));
			characteristicValueList.add(characteristicValue);
		}

		DataObject contactMedium = party.createDataObject("ContactMedium");

		List additionalCharacteristicValueList = new ArrayList();

		DataObject additionalCharacteristicValue;
		if (custInfo.getString("mobileNumber") != null) {
			additionalCharacteristicValue = putValueInCharacterList(
					"MobileNumber", custInfo.getString("mobileNumber"));
			additionalCharacteristicValueList
					.add(additionalCharacteristicValue);

		}
		if (custInfo.getString("phoneNumber") != null) {
			additionalCharacteristicValue = putValueInCharacterList(
					"PhoneNumber", custInfo.getString("phoneNumber"));
			additionalCharacteristicValueList
					.add(additionalCharacteristicValue);

		}
		if (custInfo.getString("email") != null) {
			additionalCharacteristicValue = putValueInCharacterList("Email",
					custInfo.getString("email"));
			additionalCharacteristicValueList
					.add(additionalCharacteristicValue);

		}

		contactMedium.setList("CharacteristicValue",
				additionalCharacteristicValueList);
		List rsContactMediumList = new ArrayList();

		rsContactMediumList.add(contactMedium);
		party.setList("ContactMedium", rsContactMediumList);

		party.set("CharacteristicValue", characteristicValueList);

		List partyRoleList = new ArrayList();
		partyRoleList.add(partyRole);
		productOrder.setList("PartyRole", partyRoleList);

		List productOrderItemList = new ArrayList();
		DataObject productOrderItem = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/telecom/fulfillment/schema/product_order/v3_0",
						"ProductOrderItem");
		DataObject customerAccountOrder = productOrderItem
				.createDataObject("CustomerAccountOrderItemId");
		DataObject specification = customerAccountOrder
				.createDataObject("Specification");

		if (searchCustomer.getString("orderNumber") != null) {
			specification.setString("Id", searchCustomer
					.getString("orderNumber"));

		}
		productOrderItemList.add(productOrderItem);
		productOrder.set("ProductOrderItem", productOrderItemList);
		productOrder.setString("Description", searchCustomer
				.getString("searchByOption"));
		return productOrderMessage;

	}

	public static List getOrderedProductFromOrderMessasge(
			DataObject orderMessage) {
		List orderedProductList = new ArrayList();

		DataObject order = orderMessage.getDataObject("order");
		DataObject specificationOrder = order.getDataObject("Specification");
		List orderItemList = order.getList("OrderItem");

		for (int i = 0; i < orderItemList.size(); i++) {
			DataObject orderItem = (DataObject) orderItemList.get(i);
			DataObject specification = orderItem.getDataObject("Specification");

			DataObject product = IndustrySupportUtil
					.createDO(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/telecom/usecase/DynamicOrderFulfillmentServices",
							"Product");

			product.setString("productId", specification.getString("Id"));
			product.setString("name", specification.getString("Name"));
			product.setString("categoryId", specificationOrder
					.getString("Name"));

			List characteristicValue = (List) orderItem
					.get("CharacteristicValue");
			product.setString("price", ValuesFromCharacteristics
					.firstcharacteristicValueFromCharacteristicName(
							characteristicValue, "Price"));
			orderedProductList.add(product);
		}
		return orderedProductList;
	}

	public static DataObject assignProductOrderMessageToOrderMessage(
			DataObject productOrderMessage) {
		DataObject orderMessage = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/fabric/2009/12/telecom/operations/fulfillment/schema/OrderMessage",
						"OrderMessage");

		try {
			DataObject productOrder = productOrderMessage
					.getDataObject("productOrder");
			List characteristicValueListProductOrder = productOrder
					.getList("CharacteristicValue");

			DataObject specificationProductOrder = productOrder
					.getDataObject("Specification");
			List partyRoleListProductOrder = (List) productOrder
					.get("PartyRole");
			DataObject partyProductOrder;
			DataObject partyRoleProductOrder;

			List partyRoleListOrder = new ArrayList();
			DataObject partyOrder = IndustrySupportUtil.createDO(
					"http://www.ibm.com/telecom/common/schema/party/v3_0",
					"Party");
			DataObject partyRoleOrder = IndustrySupportUtil.createDO(
					"http://www.ibm.com/telecom/common/schema/party/v3_0",
					"PartyRole");
			if (partyRoleListProductOrder.size() > 0) {
				partyRoleProductOrder = (DataObject) partyRoleListProductOrder
						.get(0);
				partyProductOrder = partyRoleProductOrder
						.getDataObject("Party");
				partyOrder = partyProductOrder;
				partyRoleOrder.setDataObject("Party", partyOrder);

			}
			partyRoleListOrder.add(partyRoleOrder);

			DataObject order = orderMessage.createDataObject("order");
			order.setString("InteractionStatus", productOrder
					.getString("InteractionStatus"));
			order.setDate("InteractionDate", productOrder
					.getDate("InteractionDate"));
			order.setDate("toTimestamp", productOrder.getDate("toTimestamp"));

			order.set("PartyRole", partyRoleListOrder);
			order.set("CharacteristicValue",
					characteristicValueListProductOrder);

			DataObject specificationOrder = order
					.createDataObject("Specification");
			specificationOrder.setString("Id", specificationProductOrder
					.getString("Id"));
			specificationOrder.setString("Name", specificationProductOrder
					.getString("Name"));

			List productOrderItemList = (List) productOrder
					.get("ProductOrderItem");
			List orderItemList = new ArrayList();

			for (int i = 0; i < productOrderItemList.size(); i++) {
				DataObject productOrderItem = (DataObject) productOrderItemList
						.get(i);
				DataObject product = productOrderItem.getDataObject("Product");
				List characteristicValue = new ArrayList();
				characteristicValue = (List) product.get("CharacteristicValue");

				DataObject orderItem = IndustrySupportUtil
						.createDO(
								"http://www.ibm.com/telecom/fulfillment/schema/order/v3_0",
								"OrderItem");
				DataObject specificationOrderItem = orderItem
						.createDataObject("Specification");
				specificationOrderItem.setString("Id", product.getString("Id"));
				specificationOrderItem.setString("Name", product
						.getString("Name"));
				orderItem.set("CharacteristicValue", characteristicValue);
				orderItemList.add(orderItem);
			}
			order.set("OrderItem", orderItemList);
		} catch (Exception e) {
			e.printStackTrace();

		}
		return orderMessage;
	}

	public static DataObject getSearchResultFromCustomerCollection(
			DataObject customerCollectionMessage) {

		List customerList = customerCollectionMessage.getList("customer");
		DataObject customer;
		DataObject searchResult = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/telecom/usecase/DynamicOrderFulfillmentServices",
						"SearchResult");
		DataObject party;

		List customerInfoList = new ArrayList();

		for (int i = 0; i < customerList.size(); i++) {
			DataObject customerInfo = IndustrySupportUtil
					.createDO(
							"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/telecom/usecase/DynamicOrderFulfillmentServices",
							"CustomerInfo");
			customer = (DataObject) customerList.get(i);
			party = customer.getDataObject("Party");

			DataObject specification = party.getDataObject("Specification");
			customerInfo.setString("accountId", specification.getString("Id"));
			customerInfo.setString("customerId", party.getString("Id"));

			List characteristicValueList = new ArrayList();
			characteristicValueList = party.getList("CharacteristicValue");

			customerInfo.setString("title", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "Title").get(0));
			customerInfo.setString("firstName", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "FirstName").get(0));
			customerInfo.setString("middleName", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "MiddleName").get(0));
			customerInfo.setString("lastName", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "LastName").get(0));
			customerInfo.setString("streetNumber", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "StreetNumber").get(0));
			customerInfo.setString("streetName", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "StreetName").get(0));
			customerInfo.setString("locality", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "Locality").get(0));
			customerInfo.setString("appartmentName", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "AppartmentName").get(0));
			customerInfo.setString("appartmentUnit", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "AppartmentUnit").get(0));
			customerInfo.setString("city", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "City").get(0));

			customerInfo.setString("state", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "State").get(0));

			customerInfo.setString("country", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "Country").get(0));

			customerInfo.setString("zipCode", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							characteristicValueList, "Zipcode").get(0));

			customerInfo.setString("preferredLanguage", "English");

			List contactMediumList = party.getList("ContactMedium");

			DataObject contactMedium = (DataObject) contactMediumList.get(0);

			List contactPersonCharacteristicValue = contactMedium
					.getList("CharacteristicValue");

			customerInfo.setString("mobileNumber", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							contactPersonCharacteristicValue, "MobileNumber")
					.get(0));

			customerInfo.setString("phoneNumber", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							contactPersonCharacteristicValue, "PhoneNumber")
					.get(0));
			customerInfo.setString("email", ValuesFromCharacteristics
					.characteristicValuesFromCharacteristicName(
							contactPersonCharacteristicValue, "Email").get(0));
			customerInfoList.add(customerInfo);
		}
		searchResult.setString("status", "Existing");
		searchResult.set("customerInfo", customerInfoList);
		return searchResult;

	}

	public static DataObject searchSelectedUser(DataObject searchResult) {

		List customerList = searchResult.getList("customerInfo");
		DataObject result = null;
		for (int i = 0; i < customerList.size(); i++) {
			DataObject customer = (DataObject) customerList.get(i);
			if (customer.getBoolean("isSelected")) {
				result = customer;
			}
		}
		return result;

	}

	public static DataObject searchSelectedProduct(
			DataObject productConfiguration) {

		DataObject productOrderMessage = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/fabric/2009/12/telecom/operations/fulfillment/schema/ProductOrderMessage",
						"ProductOrderMessage");
		DataObject productOrder = productOrderMessage
				.createDataObject("productOrder");
		DataObject specification = productOrder
				.createDataObject("Specification");
		List productOrderItemList = new ArrayList();
		long grandTotal = 0;

		List availableProductList = productConfiguration
				.getList("availableProducts");
		DataObject result = null;

		List categoryCharacteristicValueList = new ArrayList();
		for (int i = 0; i < availableProductList.size(); i++) {
			boolean isCategorySelected = false;
			DataObject availableProduct = (DataObject) availableProductList
					.get(i);
			DataObject category = availableProduct.getDataObject("category");
			List productList = category.getList("product");

			for (int j = 0; j < productList.size(); j++) {
				DataObject productOrderItem = IndustrySupportUtil
						.createDO(
								"http://www.ibm.com/telecom/fulfillment/schema/product_order/v3_0",
								"ProductOrderItem");
				DataObject productDO = productOrderItem
						.createDataObject("Product");
				List productCharacteristicValueList = new ArrayList();

				DataObject product = (DataObject) productList.get(j);
				if (product.getBoolean("isSelected")) {
					isCategorySelected = true;
					productDO.setString("Id", product.getString("productId"));
					productDO.setString("Name", product.getString("name"));
					DataObject price = putValueInCharacterList("Price", product
							.getString("price"));
					grandTotal = grandTotal
							+ Long.valueOf(product.getString("price"));
					productCharacteristicValueList.add(price);
					productDO.setList("CharacteristicValue",
							productCharacteristicValueList);
					productOrderItemList.add(productOrderItem);
				}
			}
			if (isCategorySelected) {
				specification.setString("Id", category.getString("categoryId"));
				specification.setString("Name", category.getString("name"));
			}
		}
		DataObject orderTotalBO = putValueInCharacterList("OrderTotal", String
				.valueOf(grandTotal));
		DataObject grandTotalBO = putValueInCharacterList("GrandTotal", String
				.valueOf(grandTotal));
		categoryCharacteristicValueList.add(orderTotalBO);
		categoryCharacteristicValueList.add(grandTotalBO);
		productOrder.setList("CharacteristicValue",
				categoryCharacteristicValueList);
		productOrder.setList("ProductOrderItem", productOrderItemList);
		productOrder.setString("InteractionStatus", "Open");

		/*java.util.Date date = new java.util.Date();
		
		long longDate = date.getTime() + 345600000;
		date.setTime(longDate);
		productOrder.setDate("toTimestamp", date);*/

		Calendar cal = Calendar.getInstance();
		int offset = (cal.get(Calendar.ZONE_OFFSET) + cal
				.get(Calendar.DST_OFFSET));
		long seconds = cal.getTimeInMillis();
		cal.setTimeInMillis(seconds - offset);
		Date dateInteractionDate = cal.getTime();
		productOrder.setDate("InteractionDate", dateInteractionDate);
		
		long longToTimeStamp = cal.getTimeInMillis()+345600000;
		java.util.Date dateToTimeStamp = new java.util.Date();
		dateToTimeStamp.setTime(longToTimeStamp);
		productOrder.setDate("toTimestamp", dateToTimeStamp);
		// printBO(productOrderMessage);
		return productOrderMessage;

	}

	public static DataObject mapProductCollectionMessageToProductConfiguration(
			DataObject productCollectionMessage) {
		List productOrderList = productCollectionMessage
				.getList("productOrder");
		List availableProductList = new ArrayList();
		DataObject productConfiguration = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/telecom/usecase/DynamicOrderFulfillmentServices",
						"ProductConfiguration");

		for (int a = 0; a < productOrderList.size(); a++) {

			DataObject productOrder = (DataObject) productOrderList.get(a);
			List productOrderItemList = productOrder
					.getList("ProductOrderItem");

			DataObject availableProduct = productConfiguration
					.createDataObject("availableProducts");
			DataObject categoryDO = availableProduct
					.createDataObject("category");
			categoryDO.setString("categoryId", productOrder.getString("Id"));
			categoryDO.setString("name", productOrder.getString("Name"));
			List productList = new ArrayList();
			for (int i = 0; i < productOrderItemList.size(); i++) {

				DataObject productOrderItem = (DataObject) productOrderItemList
						.get(i);
				DataObject product = productOrderItem.getDataObject("Product");

				List charecteristicValueList = product
						.getList("CharacteristicValue");
				String price = ValuesFromCharacteristics
						.characteristicValuesFromCharacteristicName(
								charecteristicValueList, "PRICE").get(0);
				DataObject productDO = IndustrySupportUtil
						.createDO(
								"http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/telecom/usecase/DynamicOrderFulfillmentServices",
								"Product");

				productDO.setString("productId", product.getString("Id"));
				productDO.setString("name", product.getString("Name"));
				productDO.setString("price", price);
				productDO.setString("categoryId", categoryDO.getString("name"));
				productList.add(productDO);
			}
			categoryDO.set("product", productList);
			availableProductList.add(availableProduct);
		}
		productConfiguration.set("availableProducts", availableProductList);
		return productConfiguration;

	}

	private static DataObject putValueInCharacterList(String feildName,
			String Value) {

		DataObject characteristicValue = IndustrySupportUtil.createDO(
				"http://www.ibm.com/telecom/common/schema/base/v3_0",
				"CharacteristicValue");
		DataObject characteristic = characteristicValue
				.createDataObject("Characteristic");
		characteristic.setString("Name", feildName);
		List valuestr = new ArrayList();
		valuestr.add(Value);
		characteristicValue.set("Value", valuestr);
		return characteristicValue;

	}

	public static String findGrandTotalForItemBasedOnState(String stateCode,
			String orderTotal) {
		float grandTotal = 0;
		String tax;
		float taxAmount;

		InputStream file = null;
		Properties properties = new Properties();
		try {
			file = TelecomCommonUtil.class
					.getResourceAsStream("TaxCode.properties");
			properties.load(file);

			tax = (String) properties.get(stateCode);
			if (tax == null) {
				tax = "0";
			}

			taxAmount = (Float.valueOf(tax)) * Integer.valueOf(orderTotal);
			taxAmount = taxAmount / 100;
			grandTotal = Float.valueOf(orderTotal) + taxAmount;
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (file != null) {
					file.close();
				}
			} catch (IOException e) {

			}
		}

		return String.valueOf(grandTotal);

	}

	static void printBO(DataObject dataObject) {
		try {
			{// print BO to log
				com.ibm.websphere.bo.BOXMLSerializer srv = (com.ibm.websphere.bo.BOXMLSerializer) new com.ibm.websphere.sca.ServiceManager()
						.locateService("com/ibm/websphere/bo/BOXMLSerializer");
				commonj.sdo.Type type = dataObject.getType();
				srv.writeDataObject(dataObject, type.getURI(), type.getName(),
						java.lang.System.out);

			}
		} catch (java.io.IOException ex) {
			ex.printStackTrace();
		}

	}

}
